"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebpackDevServer = exports.merge = exports.webpack = exports.isServe = exports.webpackArgsWithDefaults = exports.pluginExternals = exports.sharedExternals = exports.createWebConfig = exports.createElectronRendererConfig = exports.createElectronMainConfig = void 0;
const webpack_1 = require("webpack");
const webpack_merge_1 = require("webpack-merge");
const react_refresh_webpack_plugin_1 = __importDefault(require("@pmmmwh/react-refresh-webpack-plugin"));
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-expect-error no types exist for this library
const duplicate_package_checker_webpack_plugin_1 = __importDefault(require("@cerner/duplicate-package-checker-webpack-plugin"));
const path_1 = __importDefault(require("path"));
const module_1 = require("module");
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const webpack_plugin_start_electron_1 = require("./webpack-plugin-start-electron");
const args_1 = require("./args");
const loaders_1 = require("./loaders");
const util_1 = require("./util");
const externals_1 = require("./externals");
Object.defineProperty(exports, "sharedExternals", { enumerable: true, get: function () { return externals_1.sharedExternals; } });
Object.defineProperty(exports, "pluginExternals", { enumerable: true, get: function () { return externals_1.pluginExternals; } });
const webpack_plugin_multicompiler_progress_1 = require("./webpack-plugin-multicompiler-progress");
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const sharedIgnoreWarnings = [
    // Usually caused by published d.ts files pointing to non-existent ts files in
    // the ignored for publish source folder
    /Failed to parse source map.+?ENOENT/,
    // Expected in most cases for Compass
    /require function is used in a way in which dependencies cannot be statically extracted/,
    /the request of a dependency is an expression/,
    // Optional, platform-specific dependencies (mostly from driver)
    /Module not found.+?(mongo_crypt_v1.(dll|so|dylib)|@mongodb-js\/zstd|aws-crt|gcp-metadata)/,
];
const sharedResolveOptions = (target) => {
    if (typeof target === 'string') {
        target = [target];
    }
    return {
        // This replicates webpack behavior with additional special `compass:` keys
        // taking priority over the default ones that webpack uses
        //
        // See https://webpack.js.org/configuration/resolve/#resolvemainfields
        mainFields: target?.includes('web') || target?.includes('webworker')
            ? [
                'compass:browser',
                'compass:module',
                'compass:main',
                'browser',
                'module',
                'main',
            ]
            : ['compass:module', 'compass:main', 'module', 'main'],
        exportsFields: ['compass:exports', 'exports'],
        extensions: ['.jsx', '.tsx', '.ts', '...'],
        alias: {
            // Removes `browserslist` that is pulled in by `babel` and is unnecessary
            // as well as being a particularly large dependency.
            browserslist: false,
            // Removes `ampersand-sync`: `ampersand-sync` is required by `ampersand-model`,
            // but is not actually used in Compass, we don't fetch and save models via http.
            // Additionally `ampersand-sync` brings into the bundle a number of other dependencies
            // that are outdated and having known vulnerabilities.
            'ampersand-sync': false,
            // `jose` provides a browser export that uses webcrypto APIs and returns
            // webcrypto objects to represent keys, but openid-client requires
            // KeyObject instances from the Node.js crypto API (https://tinyurl.com/2rrtu2hy).
            // Manually resolve `jose` to use the Node.js export here.
            jose: require.resolve('jose'),
            // Leafygreen tries to include all the server-side emotion stuff in the
            // client bundle, this requires packaging a ton of otherwise unneccessary
            // polyfills.To work around this, we're providing a minimally required
            // polyfill for code not to break. This is mostly a problem for our web
            // packages, but also not a bad thing at all for the electron app itself.
            '@emotion/server/create-instance': path_1.default.resolve(__dirname, '..', 'polyfills', '@emotion', 'server', 'create-instance', 'index.js'),
            // This is an optional dependency of the AWS SDK that doesn't look like
            // an optional dependency to webpack because it's not wrapped in try/catch.
            '@aws-sdk/client-sso-oidc': false,
            // Some lg test helpers that are getting bundled due to re-exporting from
            // the actual component packages, never needed in the webpack bundles
            '@lg-tools/test-harnesses': false,
        },
    };
};
const providePlugin = new webpack_1.ProvidePlugin({
    URL: ['whatwg-url', 'URL'],
    URLSearchParams: ['whatwg-url', 'URLSearchParams'],
});
function createElectronMainConfig(args) {
    const opts = (0, args_1.webpackArgsWithDefaults)(args, { target: 'electron-main' });
    const namedEntry = (0, util_1.entriesToNamedEntries)(opts.entry);
    const config = {
        entry: namedEntry,
        devtool: opts.devtool,
        output: {
            path: opts.outputPath,
            filename: opts.outputFilename ?? '[name].[contenthash].main.js',
            assetModuleFilename: 'assets/[name].[hash][ext]',
            strictModuleErrorHandling: true,
            strictModuleExceptionHandling: true,
        },
        mode: opts.mode,
        target: opts.target,
        module: {
            rules: [
                (0, loaders_1.sourceMapLoader)(opts),
                (0, loaders_1.javascriptLoader)(opts),
                (0, loaders_1.nodeLoader)(opts),
                (0, loaders_1.resourceLoader)(opts),
                (0, loaders_1.sharedObjectLoader)(opts),
                (0, loaders_1.sourceLoader)(opts),
            ],
        },
        node: false,
        externals: (0, util_1.toCommonJsExternal)(externals_1.sharedExternals),
        resolve: {
            // To avoid resolving the `browser` field
            aliasFields: [],
            ...sharedResolveOptions(opts.target),
        },
        plugins: [new webpack_plugin_multicompiler_progress_1.WebpackPluginMulticompilerProgress()],
        ignoreWarnings: sharedIgnoreWarnings,
    };
    return (0, webpack_merge_1.merge)(config, opts.mode === 'development'
        ? {
            output: {
                filename: opts.outputFilename ?? '[name].main.js',
                assetModuleFilename: 'assets/[name][ext]',
            },
        }
        : {}, (0, args_1.isServe)(opts) ? { plugins: [new webpack_plugin_start_electron_1.WebpackPluginStartElectron()] } : {}, opts.analyze
        ? {
            plugins: [
                // Plugin types are not matching Webpack 5, but they work
                new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                    logLevel: 'silent',
                    analyzerPort: 'auto',
                }),
                new duplicate_package_checker_webpack_plugin_1.default(),
            ],
        }
        : {});
}
exports.createElectronMainConfig = createElectronMainConfig;
function createElectronRendererConfig(args) {
    const opts = (0, args_1.webpackArgsWithDefaults)(args, { target: 'electron-renderer' });
    const entries = (0, util_1.entriesToNamedEntries)(opts.entry);
    const config = {
        entry: entries,
        devtool: opts.devtool,
        output: {
            path: opts.outputPath,
            filename: opts.outputFilename ?? '[name].[contenthash].renderer.js',
            assetModuleFilename: 'assets/[name].[hash][ext]',
            library: opts.library ?? (0, util_1.getLibraryNameFromCwd)(opts.cwd),
            libraryTarget: 'umd',
            strictModuleErrorHandling: true,
            strictModuleExceptionHandling: true,
            globalObject: 'globalThis',
        },
        mode: opts.mode,
        target: opts.target,
        module: {
            rules: [
                (0, loaders_1.sourceMapLoader)(opts),
                (0, loaders_1.javascriptLoader)(opts),
                (0, loaders_1.nodeLoader)(opts),
                (0, loaders_1.cssLoader)(opts),
                (0, loaders_1.lessLoader)(opts),
                (0, loaders_1.assetsLoader)(opts),
                (0, loaders_1.sharedObjectLoader)(opts),
                (0, loaders_1.sourceLoader)(opts),
            ],
        },
        plugins: [
            ...(0, util_1.entriesToHtml)(entries),
            new webpack_plugin_multicompiler_progress_1.WebpackPluginMulticompilerProgress(),
            providePlugin,
        ],
        node: false,
        externals: (0, util_1.toCommonJsExternal)(externals_1.sharedExternals),
        resolve: {
            // To avoid resolving the `browser` field
            aliasFields: [],
            ...sharedResolveOptions(opts.target),
        },
        ignoreWarnings: sharedIgnoreWarnings,
    };
    return (0, webpack_merge_1.merge)(config, opts.mode === 'development'
        ? {
            output: {
                filename: opts.outputFilename ?? '[name].renderer.js',
                assetModuleFilename: 'assets/[name][ext]',
            },
        }
        : {}, opts.mode === 'production'
        ? {
            plugins: [
                new mini_css_extract_plugin_1.default(),
            ],
        }
        : {}, (0, args_1.isServe)(opts)
        ? {
            devServer: {
                magicHtml: false,
                port: opts.devServerPort,
                devMiddleware: {
                    // It's slower than in-memory fs, but required so that we can
                    // start the electron app
                    writeToDisk: true,
                },
                client: {
                    overlay: {
                        errors: true,
                        warnings: false,
                    },
                },
                https: false,
                hot: opts.hot,
            },
            plugins: [
                new webpack_plugin_start_electron_1.WebpackPluginStartElectron(),
            ].concat(opts.hot
                ? [
                    // Plugin types are not matching Webpack 5, but they work
                    new react_refresh_webpack_plugin_1.default(),
                ]
                : []),
        }
        : {}, opts.analyze
        ? {
            plugins: [
                // Plugin types are not matching Webpack 5, but they work
                new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                    logLevel: 'silent',
                    analyzerPort: 'auto',
                }),
                new duplicate_package_checker_webpack_plugin_1.default(),
            ],
        }
        : {});
}
exports.createElectronRendererConfig = createElectronRendererConfig;
function createWebConfig(args) {
    const opts = (0, args_1.webpackArgsWithDefaults)(args, { target: 'web' });
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    const { peerDependencies } = require(path_1.default.join(opts.cwd, 'package.json'));
    return {
        entry: (0, util_1.entriesToNamedEntries)(opts.entry),
        devtool: opts.devtool,
        output: {
            path: opts.outputPath,
            filename: opts.outputFilename ?? '[name].js',
            assetModuleFilename: 'assets/[name][ext]',
            library: opts.library ?? (0, util_1.getLibraryNameFromCwd)(opts.cwd),
            libraryTarget: 'umd',
            // These two options are subtly different, and while
            // `strictModuleExceptionHandling` is deprecated, it is the only
            // one that actually gives us the right behavior currently.
            // https://github.com/webpack/webpack/blob/3ad4fcac25a976277f2d9cceb37bc81602e96b13/lib/javascript/JavascriptModulesPlugin.js#L1326-L1346
            // Note that hot module reloading turns these on by default,
            // so this is only affecting production builds and not the
            // typical development mode that we work in.
            strictModuleErrorHandling: true,
            strictModuleExceptionHandling: true,
            globalObject: 'globalThis',
        },
        mode: opts.mode,
        target: opts.target,
        module: {
            rules: [
                (0, loaders_1.sourceMapLoader)(opts),
                (0, loaders_1.javascriptLoader)(opts, true),
                (0, loaders_1.nodeLoader)(opts),
                (0, loaders_1.cssLoader)(opts, true),
                (0, loaders_1.lessLoader)(opts),
                (0, loaders_1.assetsLoader)(opts),
                (0, loaders_1.sourceLoader)(opts),
            ],
        },
        // This follows current Compass plugin behavior and is here more or less to
        // keep compat for the external plugin users
        externals: {
            ...(0, util_1.toCommonJsExternal)(externals_1.sharedExternals),
            ...(0, util_1.toCommonJsExternal)(Object.keys(peerDependencies ?? {})),
            ...(0, util_1.toCommonJsExternal)(module_1.builtinModules.flatMap((m) => [m, `node:${m}`])),
        },
        resolve: {
            ...sharedResolveOptions(opts.target),
        },
        ignoreWarnings: sharedIgnoreWarnings,
        plugins: [
            providePlugin,
            ...((0, args_1.isServe)(opts) && opts.hot
                ? [
                    // Plugin types are not matching Webpack 5, but they work
                    new react_refresh_webpack_plugin_1.default(),
                ]
                : opts.analyze
                    ? [
                        // Plugin types are not matching Webpack 5, but they work
                        new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                            logLevel: 'silent',
                            analyzerPort: 'auto',
                        }),
                        new duplicate_package_checker_webpack_plugin_1.default(),
                    ]
                    : []),
        ],
    };
}
exports.createWebConfig = createWebConfig;
var args_2 = require("./args");
Object.defineProperty(exports, "webpackArgsWithDefaults", { enumerable: true, get: function () { return args_2.webpackArgsWithDefaults; } });
Object.defineProperty(exports, "isServe", { enumerable: true, get: function () { return args_2.isServe; } });
var webpack_2 = require("webpack");
Object.defineProperty(exports, "webpack", { enumerable: true, get: function () { return __importDefault(webpack_2).default; } });
var webpack_merge_2 = require("webpack-merge");
Object.defineProperty(exports, "merge", { enumerable: true, get: function () { return webpack_merge_2.merge; } });
var webpack_dev_server_1 = require("webpack-dev-server");
Object.defineProperty(exports, "WebpackDevServer", { enumerable: true, get: function () { return __importDefault(webpack_dev_server_1).default; } });
//# sourceMappingURL=index.js.map